{ **************************************************************************** }
{ *                                                                          * }
{ * DUNGEONS AND DRAGONS (Mattel, Action Arcade Series) Simulator            * }
{ * by Luca "MADrigal" Antignano, February 2001 (revision: August 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;

    btn_cursor_down:          TImage;       btn_cursor_top:           TImage;
    btn_attack_down:          TImage;       btn_attack_top:           TImage;
    btn_acl_down:             TImage;       btn_acl_top:              TImage;
    btn_move_down:            TImage;       btn_move_top:             TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (alarm, time/score, game a/b, etc.)
    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage;

    im_panel_number_1_0: TImage; im_panel_number_1_1: TImage; im_panel_number_1_2: TImage; im_panel_number_1_3: TImage; im_panel_number_1_4: TImage;
    im_panel_number_1_5: TImage; im_panel_number_1_6: TImage; im_panel_number_1_7: TImage; im_panel_number_1_8: TImage; im_panel_number_1_9: TImage;
    im_panel_number_1_a: TImage; im_panel_number_1_b: TImage; im_panel_number_1_c: TImage; im_panel_number_1_d: TImage; im_panel_number_1_e: TImage;
    im_panel_number_1_f: TImage; im_panel_number_1_h: TImage; im_panel_number_1_j: TImage; im_panel_number_1_l: TImage;

    im_panel_number_2_0: TImage; im_panel_number_2_1: TImage; im_panel_number_2_2: TImage; im_panel_number_2_3: TImage; im_panel_number_2_4: TImage;
    im_panel_number_2_5: TImage; im_panel_number_2_6: TImage; im_panel_number_2_7: TImage; im_panel_number_2_8: TImage; im_panel_number_2_9: TImage;

    // top panel (bats, warriors, arrows)
                          im_bat_1: TImage;     im_bat_2: TImage;     im_bat_3: TImage;     im_bat_4: TImage;     im_bat_5: TImage;
    im_warrior_0: TImage; im_warrior_1: TImage; im_warrior_2: TImage; im_warrior_3: TImage; im_warrior_4: TImage; im_warrior_5: TImage; im_warrior_6: TImage; im_warrior_7: TImage;
    im_arrow_0: TImage;   im_arrow_1: TImage;   im_arrow_2: TImage;   im_arrow_3: TImage;   im_arrow_4: TImage;   im_arrow_5: TImage; im_arrow_6: TImage;

    // middle panel (cursor, maze + warrior)
    im_cursor_up: TImage; im_cursor_left: TImage; im_cursor_right: TImage; im_cursor_down: TImage;
    im_maze: TImage;

    // right panel (pit, hook, dragon)
    im_pit: TImage; im_hook: TImage; im_dragon: TImage;

    // bottom panel (icons)
    im_icon_arrow: TImage; im_icon_hook: TImage;

    // cheat mode
    la_cheat_0: TLabel; la_cheat_1: TLabel; la_cheat_2: TLabel; la_cheat_3: TLabel; la_cheat_4: TLabel; la_cheat_5: TLabel; la_cheat_6: TLabel; la_cheat_7: TLabel; la_cheat_8: TLabel; la_cheat_9: TLabel;

{ * TIMERS                                                                   * }
    Timer_Bat_Moves:        TTimer;
    Timer_Bat_Warrior_Move: TTimer;
    Timer_Blinking:         TTimer;
    Timer_Demo_Mode:        TTimer;
    Timer_Five_Seconds:     TTimer;
    Timer_Found_Arrow_Hook: TTimer;
    Timer_Found_Dragon:     TTimer;
    Timer_Game_Over_Lose:   TTimer;
    Timer_Game_Over_Win:    TTimer;
    Timer_Game_Start:       TTimer;
    Timer_Move_To_Arrow:    TTimer;
    Timer_Move_To_Bat:      TTimer;
    Timer_Move_To_Dragon:   TTimer;
    Timer_Move_to_Free:     TTimer;
    Timer_Move_To_Hook:     TTimer;
    Timer_Move_To_Pit:      TTimer;
    Timer_Score:            TTimer;
    Timer_Shoot_Arrow:      TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_Cheat                (b: boolean);
    procedure Vid_Show_Cursor               (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_cursor_downMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_cursor_downMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_attack_downMouseDown      (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_attack_downMouseUp        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_move_downMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_move_downMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_cursor_press;             procedure Inp_cursor_release;
    procedure Inp_attack_press;             procedure Inp_attack_release;
    procedure Inp_move_press;               procedure Inp_move_release;

    procedure Inp_acl_press;                procedure Inp_acl_release;

    procedure Inp_cursor_up_press;
    procedure Inp_cursor_left_press;
    procedure Inp_cursor_right_press;
    procedure Inp_cursor_down_press;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Close_To_Arrow;
    procedure Gam_Check_Close_To_Bat;
    procedure Gam_Check_Close_To_Dragon;
    procedure Gam_Check_Close_To_Hook;
    procedure Gam_Check_Close_To_Pit;
    procedure Gam_Set_Item_Position         (p: integer);
    procedure Gam_Set_Mode;
    procedure Gam_Set_Warriors_Cell;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Bat_MovesTimer          (Sender: TObject);
    procedure Timer_Bat_Warrior_MoveTimer   (Sender: TObject);
    procedure Timer_BlinkingTimer           (Sender: TObject);
    procedure Timer_Demo_ModeTimer          (Sender: TObject);
    procedure Timer_Five_SecondsTimer       (Sender: TObject);
    procedure Timer_Found_Arrow_HookTimer   (Sender: TObject);
    procedure Timer_Found_DragonTimer       (Sender: TObject);
    procedure Timer_Game_Over_LoseTimer     (Sender: TObject);
    procedure Timer_Game_Over_WinTimer      (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Move_To_ArrowTimer      (Sender: TObject);
    procedure Timer_Move_To_BatTimer        (Sender: TObject);
    procedure Timer_Move_To_DragonTimer     (Sender: TObject);
    procedure Timer_Move_to_FreeTimer       (Sender: TObject);
    procedure Timer_Move_To_HookTimer       (Sender: TObject);
    procedure Timer_Move_To_PitTimer        (Sender: TObject);
    procedure Timer_ScoreTimer              (Sender: TObject);
    procedure Timer_Shoot_ArrowTimer        (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Dungeons & Dragons (Mattel Electronics, Action Arcade Series)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\Dnd50';                                      // registry key

  HINT_CLICK  = 'click or press';
  HINT_ATTACK = 'to start game/attack';
  HINT_CURSOR = 'or use the arrow keys to select a direction';
  HINT_MOVE   = 'to select game/move';
  HINT_ACL    = 'to reset the game';

{}MENU_ITEMS = 12;
  NUM_KEYS = 4;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

  pfs_arrow,
  pfs_bat,
  pfs_cursor,
  pfs_dragon,
  pfs_found,
  pfs_gameover,
  pfs_got,
  pfs_pit,
  pfs_step,
  pfs_step_bat,
  pfs_win:                 PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..2]           of TImage;                     // empty sprites to host score/messages digits
  aIm_panel_number:        array[1..2, 0..9]     of TImage;                     // numbers (left and right digit)
  aIm_panel_letter:        array[0..10]          of TImage;                     // letters (left digit only) (A-J plus L)

  aIm_cursor:              array[1..4]           of TImage;                     // up / right / down / left

  aIm_bat:                 array[1..5]           of TImage;                     // 1-5 (top panel)
  aIm_warrior:             array[0..7]           of TImage;                     // 0-5 (top panel), 6-7 (in the pit)
  aIm_arrow:               array[0..6]           of TImage;                     // 0-4 (top panel), 5 (missed shot), 6 (arrow in dragon)

{ * arrays of numbers and records * }
  aMaze:                   array[-1..10, -1..10] of integer;                    // 0=empty; 1=warrior; 2=pit; 3=bat 4=hook; 5=arrow; 6=dragon; 7=warrior in a pit
  rBat:                    array[1..4]           of record                      // bats
                                                      x, y: integer;              // x=1-10; y=1-10
                                                    end;
  aCheat:                  array[0..9]           of TLabel;                     // cheat mode maze (panel)

{ * arrays of mixed types * }
  aKeys:                   array[1..NUM_KEYS] of record                         // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..40] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL/1; 1=ACL/2 (DEMO); 2/3/4=SELECT L1/L2/L3; 5/6/7=GAME L1/L2/L3; 8=GAME OVER
  iGameOverMode,                                                                // 1/2/3=GAME OVER Le/L2/L3
  iWarriorX, iWarriorY,                                                         // x=1-10; y=1-10
  iCursor,                                                                      // 1=N; 2=E; 3=S; 4=W
  iScore
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bController,
  bCursorPressed,
  bAttackPressed,
  bMovePressed,

  bHaveArrow,
  bHaveHook,
  bBatHasWarrior,

  bCloseToArrow,
  bCloseToBat,
  bCloseToDragon,
  bCloseToHook,
  bCloseToPit,

  bFiveSeconds,                                                                 // to be used in "game over" mode (Attack button pressed for 5 seconds)

  bCheat                                                                        // cheat mode enabled/disabled
                         : boolean;

  // Game variables: loops
  iLoopBatWarriorMove,
  iLoopDemoMode,
  iLoopFoundArrowHook,
  iLoopFoundDragon,
  iLoopGameOver,
  iLoopMoveToArrow,
  iLoopMoveToBat,
  iLoopMoveToDragon,
  iLoopMoveToFree,
  iLoopMoveToHook,
  iLoopMoveToPit,
  iLoopScore,
  iLoopShootArrow
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2;

  // numbers and letters
  aIm_panel_number[1, 0] := im_panel_number_1_0; aIm_panel_number[1, 1] := im_panel_number_1_1; aIm_panel_number[1, 2] := im_panel_number_1_2; aIm_panel_number[1, 3] := im_panel_number_1_3; aIm_panel_number[1, 4] := im_panel_number_1_4;
  aIm_panel_number[1, 5] := im_panel_number_1_5; aIm_panel_number[1, 6] := im_panel_number_1_6; aIm_panel_number[1, 7] := im_panel_number_1_7; aIm_panel_number[1, 8] := im_panel_number_1_8; aIm_panel_number[1, 9] := im_panel_number_1_9;

  aIm_panel_number[2, 0] := im_panel_number_2_0; aIm_panel_number[2, 1] := im_panel_number_2_1; aIm_panel_number[2, 2] := im_panel_number_2_2; aIm_panel_number[2, 3] := im_panel_number_2_3; aIm_panel_number[2, 4] := im_panel_number_2_4;
  aIm_panel_number[2, 5] := im_panel_number_2_5; aIm_panel_number[2, 6] := im_panel_number_2_6; aIm_panel_number[2, 7] := im_panel_number_2_7; aIm_panel_number[2, 8] := im_panel_number_2_8; aIm_panel_number[2, 9] := im_panel_number_2_9;

  aIm_panel_letter[0] := im_panel_number_1_a; aIm_panel_letter[1] := im_panel_number_1_b; aIm_panel_letter[2] := im_panel_number_1_c; aIm_panel_letter[3] := im_panel_number_1_d; aIm_panel_letter[4] := im_panel_number_1_e; aIm_panel_letter[5] := im_panel_number_1_f;
  aIm_panel_letter[6] := im_panel_number_1_6; aIm_panel_letter[7] := im_panel_number_1_h; aIm_panel_letter[8] := im_panel_number_1_1; aIm_panel_letter[9] := im_panel_number_1_j; aIm_panel_letter[10] := im_panel_number_1_l;

  // cursor panel
  aIm_cursor[1] := im_cursor_up;
  aIm_cursor[2] := im_cursor_right;
  aIm_cursor[3] := im_cursor_down;
  aIm_cursor[4] := im_cursor_left;

  // top panel (bats, warriors and arrows)
                                  aIm_bat[1] := im_bat_1;         aIm_bat[2] := im_bat_2;         aIm_bat[3] := im_bat_3;         aIm_bat[4] := im_bat_4;         aIm_bat[5] := im_bat_5;
  aIm_warrior[0] := im_warrior_0; aIm_warrior[1] := im_warrior_1; aIm_warrior[2] := im_warrior_2; aIm_warrior[3] := im_warrior_3; aIm_warrior[4] := im_warrior_4; aIm_warrior[5] := im_warrior_5; aIm_warrior[6] := im_warrior_6; aIm_warrior[7] := im_warrior_7;
  aIm_arrow[0] := im_arrow_0;     aIm_arrow[1] := im_arrow_1;     aIm_arrow[2] := im_arrow_2;     aIm_arrow[3] := im_arrow_3;     aIm_arrow[4] := im_arrow_4;     aIm_arrow[5] := im_arrow_5;     aIm_arrow[6] := im_arrow_6;

  // cheat mode
  aCheat[0] := la_cheat_0; aCheat[1] := la_cheat_1; aCheat[2] := la_cheat_2; aCheat[3] := la_cheat_3; aCheat[4] := la_cheat_4; aCheat[5] := la_cheat_5; aCheat[6] := la_cheat_6; aCheat[7] := la_cheat_7; aCheat[8] := la_cheat_8; aCheat[9] := la_cheat_9;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''alt'' key';
  aKeyHints[38] := '''ctrl'' key';
  aKeyHints[39] := '''shift'' key';
  aKeyHints[40] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 6);                              // "ACL"    (default: "5")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 39);                             // "Cursor" (default: "Shift")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 38);                             // "Start/Attack" (default: "Ctrl")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 37);                             // "Move" (default: "Alt")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to NUM_KEYS do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_ACL;     // "ACL"

  btn_cursor_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_CURSOR;  // "Cursor"
  btn_attack_top.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_ATTACK;  // "Start/Attack"
  btn_move_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_MOVE;    // "Move"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_9_n2;
{}aMenuItem[9]  := pm_10_fullscreen;
{}aMenuItem[10] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[11] := pm_13_n3;
{}aMenuItem[12] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  bCheat := false;

  bController    := false;                                                      // prevents hero movement
  bCursorPressed := false;                                                      // cursor key is not currently being kept pressed
  bAttackPressed := false;                                                      // attack key is not currently being kept pressed
  bMovePressed   := false;                                                      // move key is not currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_arrow);  FSOUND_Sample_Free(pfs_bat);
  FSOUND_Sample_Free(pfs_cursor); FSOUND_Sample_Free(pfs_dragon);
  FSOUND_Sample_Free(pfs_found);  FSOUND_Sample_Free(pfs_gameover);
  FSOUND_Sample_Free(pfs_got);    FSOUND_Sample_Free(pfs_pit);
  FSOUND_Sample_Free(pfs_step);   FSOUND_Sample_Free(pfs_step_bat);
  FSOUND_Sample_Free(pfs_win);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 5 do aIm_bat[i].Visible     := b;
  for i := 0 to 7 do aIm_warrior[i].Visible := b;
  for i := 0 to 6 do aIm_arrow[i].Visible   := b;
  for i := 1 to 4 do aIm_cursor[i].Visible  := b;

  for i := 1 to 2 do
    begin
      aIm_panel_digit_pos[i].Visible := true;

      if b then
        aIm_panel_digit_pos[i].Picture := aIm_panel_number[i, 8].Picture
      else
        aIm_panel_digit_pos[i].Picture := nil;
    end;

  im_pit.Visible        := b;
  im_maze.Visible       := b;
  im_hook.Visible       := b;
  im_icon_arrow.Visible := b;
  im_icon_hook.Visible  := b;
  im_dragon.Visible     := b;
end;





procedure TForm1.Vid_Show_Cheat(b: boolean);
{ displays/hides current status of the maze                                    }
  var
    i, j: integer;
begin
  for i := 0 to 9 do aCheat[i].Caption := '';

  if b then
    for i := 0 to 9 do
      for j := 0 to 9 do
        begin
          aCheat[i].Caption := aCheat[i].Caption + inttostr(aMaze[j, i]);
          aCheat[i].Left := im_background.Left + im_background.Width  - aCheat[i].Width - 10;
          aCheat[i].Top  := im_background.Top  + im_background.Height - (aCheat[i].Height * (10 - i)) - 10;
        end;
end;





procedure TForm1.Vid_Show_Cursor(var p: integer; b: boolean);
{ displays/hides cursor in 'p' position                                        }
begin
  aIm_cursor[p].Visible := b;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_press;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_release;
end;





procedure TForm1.btn_cursor_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ CURSOR button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_cursor_press;
end;





procedure TForm1.btn_cursor_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ CURSOR button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_cursor_release;
end;





procedure TForm1.btn_attack_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ATTACK button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_attack_press;
end;





procedure TForm1.btn_attack_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ATTACK button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_attack_release;
end;





procedure TForm1.btn_move_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ MOVE button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_move_press;
end;





procedure TForm1.btn_move_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ MOVE button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_move_release;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_acl_press          else                       // ACL

  if Key = aKeys[2].word then Inp_cursor_press       else                       // Cursor
  if Key = aKeys[3].word then Inp_attack_press       else                       // Start/Attack
  if Key = aKeys[4].word then Inp_move_press         else                       // Move

  if Key = VK_UP         then Inp_cursor_up_press    else                       // 'up'    -> SETS CURSOR UP
  if Key = VK_LEFT       then Inp_cursor_left_press  else                       // 'left'  -> SETS CURSOR LEFT
  if Key = VK_RIGHT      then Inp_cursor_right_press else                       // 'right' -> SETS CURSOR RIGHT
  if Key = VK_DOWN       then Inp_cursor_down_press  else                       // 'down'  -> SETS CURSOR DOWN

  if Key = VK_INSERT     then                                                   // 'INSERT" -> DEBUG/CHEAT MODE
    if (iMode >= 5) and (iMode <= 7) then
      begin
        bCheat := not bCheat;
        Vid_Show_Cheat(bCheat);
      end;
end;





procedure TForm1.FormKeyUp(Sender:TObject; var Key:Word; Shift:TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_acl_release    else                           // ACL

  if Key = aKeys[2].word then Inp_cursor_release else                           // Cursor
  if Key = aKeys[3].word then Inp_attack_release else                           // Start/Attack
  if Key = aKeys[4].word then Inp_move_release   else                           // Move

  if Key = VK_UP         then Inp_cursor_release else                           // 'up'    -> SETS CURSOR UP
  if Key = VK_LEFT       then Inp_cursor_release else                           // 'left'  -> SETS CURSOR LEFT
  if Key = VK_RIGHT      then Inp_cursor_release else                           // 'right' -> SETS CURSOR RIGHT
  if Key = VK_DOWN       then Inp_cursor_release;                               // 'down'  -> SETS CURSOR DOWN
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_cursor_press;
{ CURSOR button: PRESS                                                         }
begin
  btn_cursor_down.Visible := true;

  if not bCursorPressed then
    begin
      bCursorPressed := true;

      if (iMode >= 5) and (iMode <= 7)
        and bController then
          begin
            Vid_Show_Cursor(iCursor, false);
            Inc(iCursor);

            if iCursor = 5 then
              iCursor := 1;

            Vid_Show_Cursor(iCursor, true);

            Snd_Play(pfs_cursor);
          end;
    end;
end;





procedure TForm1.Inp_cursor_release;
{ CURSOR button: RELEASE                                                       }
begin
  btn_cursor_down.Visible := false;
  bCursorPressed := false;
end;





procedure TForm1.Inp_attack_press;
{ ATTACK button: PRESS                                                         }
begin
  btn_attack_down.Visible := true;

  if not bAttackPressed then
    begin
      bAttackPressed := true;

      case iMode of
        1: begin                                                                // ACL/2 (DEMO) ->
             iMode := 2; Gam_Set_Mode;                                          //              -> SELECT MODE (L1)
           end;
        2, 3, 4: begin                                                          // SELECT MODE (L1/L2/L3) ->
             Inc(iMode, 3); Gam_Set_Mode;                                       //                        -> GAME MODE (L1/L2/L3)
           end;
        5, 6, 7: begin                                                          // GAME MODE (L1/L2/L3) ->
             if bController and bHaveArrow then                                 //                      -> PERFORMS ACTION
               begin
                 bController := false;                                            // shoots the arrow

                 Gam_Stop_All_Timers;
                 Vid_Show_All(false);

                 im_icon_hook.Visible := bHaveHook;
                 aIm_warrior[0].Visible := true;
                 aIm_arrow[0].Visible := true;
                 Snd_Play(pfs_arrow);

                 iLoopShootArrow := 1;
                 Timer_Shoot_Arrow.Interval := 315;
                 Timer_Shoot_Arrow.Enabled := true;
               end;
           end;
        8: begin                                                                // GAME OVER ->
             bFiveSeconds := false;
             Timer_Five_Seconds.Interval := 5000;
             Timer_Five_Seconds.Enabled := true;
           end;
      end; // case..of
    end;
end;





procedure TForm1.Inp_attack_release;
{ ATTACK button: RELEASE                                                       }
begin
  btn_attack_down.Visible := false;
  bAttackPressed := false;

  // special case if the button is pressed in "Game Over" mode
  if iMode = 8 then
    begin
      Timer_Five_Seconds.Enabled := false;

      if not bFiveSeconds then
        begin
          iMode := 2; Gam_Set_Mode;
        end;
    end;
end;





procedure TForm1.Inp_move_press;
{ MOVE button: PRESS                                                           }
begin
  btn_move_down.Visible := true;

  if not bMovePressed then
    begin
      bMovePressed := true;

      if (iMode >= 2) and (iMode <= 4) then                                     // SELECT MODE (L1/L2/L3) ->
        begin                                                                   //                        -> LOOPS SELECTION
          Inc(iMode);

          if iMode = 5 then
            iMode := 2;

          Gam_Set_Mode;
        end;

      if (iMode >= 5) and (iMode <= 7)                                          // GAME MODE (L1/L2/L3) ->
        and bController then                                                    //                      -> MOVES WARRIOR
        begin
          bController := false;

          // manages cell value
          case aMaze[iWarriorX, iWarriorY] of
            1: aMaze[iWarriorX, iWarriorY] := 0; { moves to an empty cell: resets value to 0 }
            7: aMaze[iWarriorX, iWarriorY] := 2; { moves to a pit:         resets value to 2 }
          end;

          // moves warrior
          case iCursor of
            1: begin Dec(iWarriorY); if iWarriorY < 0 then iWarriorY := 9; end;
            2: begin Inc(iWarriorX); if iWarriorX > 9 then iWarriorX := 0; end;
            3: begin Inc(iWarriorY); if iWarriorY > 9 then iWarriorY := 0; end;
            4: begin Dec(iWarriorX); if iWarriorX < 0 then iWarriorX := 9; end;
          end; // case..of

          // movement animation starts here
          Gam_Stop_All_Timers;
          Vid_Show_All(false);
          im_icon_hook.Visible := bHaveHook;
          im_icon_arrow.Visible := bHaveArrow;
          aIm_warrior[1].Visible := true;

          Snd_Play(pfs_step);

          // checks which kind of cell warrior is moving to
          case aMaze[iWarriorX, iWarriorY] of
            0: begin                                iLoopMoveToFree   := 1; Timer_Move_to_Free.Interval   := 315; Timer_Move_to_Free.Enabled   := true; end; // free:   moves
            2: begin                                iLoopMoveToPit    := 1; Timer_Move_To_Pit.Interval    := 315; Timer_Move_To_Pit.Enabled    := true; end; // pit:    moves and falls
            3: begin aIm_bat[1].Visible   := false; iLoopMoveToBat    := 1; Timer_Move_To_Bat.Interval    := 315; Timer_Move_To_Bat.Enabled    := true; end; // bat:    moves and flies away
            4: begin im_hook.Visible      := false; iLoopMoveToHook   := 1; Timer_Move_To_Hook.Interval   := 315; Timer_Move_To_Hook.Enabled   := true; end; // hook:   moves and takes it
            5: begin aIm_arrow[1].Visible := false; iLoopMoveToArrow  := 1; Timer_Move_To_Arrow.Interval  := 315; Timer_Move_To_Arrow.Enabled  := true; end; // arrow:  moves and takes it
            6: begin im_dragon.Visible    := false; iLoopMoveToDragon := 1; Timer_Move_To_Dragon.Interval := 315; Timer_Move_To_Dragon.Enabled := true; end; // dragon: moves and gets killed
          end; // case..of
        end;
    end;
end;





procedure TForm1.Inp_move_release;
{ MOVE button: RELEASE                                                         }
begin
  btn_move_down.Visible := false;
  bMovePressed := false;
end;





procedure TForm1.Inp_acl_press;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  iMode := 0; Gam_Set_Mode;                                                     // any mode -> ACL/1
end;





procedure TForm1.Inp_acl_release;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  iMode := 1; Gam_Set_Mode;                                                     // ACL/1 -> ACL/2 (DEMO)
end;





procedure TForm1.Inp_cursor_up_press;
{ UP key: PRESS                                                                }
begin
  btn_cursor_down.Visible := true;

  if not bCursorPressed then
    begin
      bCursorPressed := true;

      if (iMode >= 5) and (iMode <= 7)
        and bController then
        begin
          Vid_Show_Cursor(iCursor, false);
          iCursor := 1;
          Vid_Show_Cursor(iCursor, true);

          Snd_Play(pfs_cursor);
        end;
    end;
end;





procedure TForm1.Inp_cursor_left_press;
{ LEFT key: PRESS                                                              }
begin
  btn_cursor_down.Visible := true;

  if not bCursorPressed then
    begin
      bCursorPressed := true;

      if (iMode >= 5) and (iMode <= 7)
        and bController then
        begin
          Vid_Show_Cursor(iCursor, false);
          iCursor := 4;
          Vid_Show_Cursor(iCursor, true);

          Snd_Play(pfs_cursor);
        end;
    end;
end;





procedure TForm1.Inp_cursor_right_press;
{ RIGHT key: PRESS                                                             }
begin
  btn_cursor_down.Visible := true;

  if not bCursorPressed then
    begin
      bCursorPressed := true;

      if (iMode >= 5) and (iMode <= 7)
        and bController then
        begin
          Vid_Show_Cursor(iCursor, false);
          iCursor := 2;
          Vid_Show_Cursor(iCursor, true);

          Snd_Play(pfs_cursor);
        end;
    end;
end;





procedure TForm1.Inp_cursor_down_press;
{ DOWN key: PRESS                                                              }
begin
  btn_cursor_down.Visible := true;

  if not bCursorPressed then
    begin
      bCursorPressed := true;

      if (iMode >= 5) and (iMode <= 7)
        and bController then
        begin
          Vid_Show_Cursor(iCursor, false);
          iCursor := 3;
          Vid_Show_Cursor(iCursor, true);

          Snd_Play(pfs_cursor);
        end;
    end;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all timers at once                                                     }
begin
  Timer_Bat_Moves.Enabled        := false;
  Timer_Bat_Warrior_Move.Enabled := false;
  Timer_Blinking.Enabled         := false;
  Timer_Demo_Mode.Enabled        := false;
  Timer_Five_Seconds.Enabled     := false;
  Timer_Found_Arrow_Hook.Enabled := false;
  Timer_Found_Dragon.Enabled     := false;
  Timer_Game_Over_Lose.Enabled   := false;
  Timer_Game_Over_Win.Enabled    := false;
  Timer_Game_Start.Enabled       := false;
  Timer_Move_To_Arrow.Enabled    := false;
  Timer_Move_To_Bat.Enabled      := false;
  Timer_Move_To_Dragon.Enabled   := false;
  Timer_Move_to_Free.Enabled     := false;
  Timer_Move_To_Hook.Enabled     := false;
  Timer_Move_To_Pit.Enabled      := false;
  Timer_Score.Enabled            := false;
  Timer_Shoot_Arrow.Enabled      := false;
end;





procedure TForm1.Gam_Check_Close_To_Arrow;
{ checks whether the warrior is close to the arrow                             }
  var
    i: integer;
    bCheck: boolean;
begin
  bCheck := false;

  i := iWarriorX - 1; if i < 0 then i := 9; if aMaze[i, iWarriorY] = 5 then bCheck := true;
  i := iWarriorX + 1; if i > 9 then i := 0; if aMaze[i, iWarriorY] = 5 then bCheck := true;
  i := iWarriorY - 1; if i < 0 then i := 9; if aMaze[iWarriorX, i] = 5 then bCheck := true;
  i := iWarriorY + 1; if i > 9 then i := 0; if aMaze[iWarriorX, i] = 5 then bCheck := true;

  if bCheck then
    begin
      bCloseToArrow := true;
      aIm_arrow[1].Visible := true;
    end
  else
    bCloseToArrow := false;
end;





procedure TForm1.Gam_Check_Close_To_Bat;
{ checks whether the warrior is close to a bat                                 }
  var
    i:integer;
    bCheck: boolean;
begin
  bCheck := false;

  i := iWarriorX - 1; if i < 0 then i := 9; if aMaze[i, iWarriorY] = 3 then bCheck := true;
  i := iWarriorX + 1; if i > 9 then i := 0; if aMaze[i, iWarriorY] = 3 then bCheck := true;
  i := iWarriorY - 1; if i < 0 then i := 9; if aMaze[iWarriorX, i] = 3 then bCheck := true;
  i := iWarriorY + 1; if i > 9 then i := 0; if aMaze[iWarriorX, i] = 3 then bCheck := true;

  if bCheck then
    begin
      bCloseToBat := true;
      aIm_bat[1].Visible := true;
    end
  else
    bCloseToBat := false;
end;





procedure TForm1.Gam_Check_Close_To_Dragon;
{ checks whether the warrior is close to the dragon                            }
  var
    i: integer;
    bCheck: boolean;
begin
  bCheck := false;

  i := iWarriorX - 1; if i < 0 then i := 9; if aMaze[i, iWarriorY] = 6 then bCheck := true;
  i := iWarriorX + 1; if i > 9 then i := 0; if aMaze[i, iWarriorY] = 6 then bCheck := true;
  i := iWarriorY - 1; if i < 0 then i := 9; if aMaze[iWarriorX, i] = 6 then bCheck := true;
  i := iWarriorY + 1; if i > 9 then i := 0; if aMaze[iWarriorX, i] = 6 then bCheck := true;

  if bCheck then
    begin
      bCloseToDragon := true;
      im_dragon.Visible := true;
    end
  else
    bCloseToDragon := false;
end;





procedure TForm1.Gam_Check_Close_To_Hook;
{ checks whether the warrior is close to the hook                              }
  var
    i: integer;
    bCheck: boolean;
begin
  bCheck := false;

  i := iWarriorX - 1; if i < 0 then i := 9; if aMaze[i, iWarriorY] = 4 then bCheck := true;
  i := iWarriorX + 1; if i > 9 then i := 0; if aMaze[i, iWarriorY] = 4 then bCheck := true;
  i := iWarriorY - 1; if i < 0 then i := 9; if aMaze[iWarriorX, i] = 4 then bCheck := true;
  i := iWarriorY + 1; if i > 9 then i := 0; if aMaze[iWarriorX, i] = 4 then bCheck := true;

  if bCheck then
    begin
      bCloseToHook := true;
      im_hook.Visible := true;
    end
  else
    bCloseToHook := false;
end;





procedure TForm1.Gam_Check_Close_To_Pit;
{ checks whether the warrior is close to a pit                                 }
  var
    i: integer;
    bCheck: boolean;
begin
  bCheck := false;

  i := iWarriorX - 1; if i < 0 then i := 9; if aMaze[i, iWarriorY] = 2 then bCheck := true;
  i := iWarriorX + 1; if i > 9 then i := 0; if aMaze[i, iWarriorY] = 2 then bCheck := true;
  i := iWarriorY - 1; if i < 0 then i := 9; if aMaze[iWarriorX, i] = 2 then bCheck := true;
  i := iWarriorY + 1; if i > 9 then i := 0; if aMaze[iWarriorX, i] = 2 then bCheck := true;

  if bCheck then
    begin
      bCloseToPit := true;
      im_pit.Visible := true;
    end
  else
    bCloseToPit := false;
end;





procedure TForm1.Gam_Set_Item_Position(p: integer);
{ sets new coordinates for hook (p=4), arrow (p=5) and dragon (p=6)            }
  var
    i, j, x, y: integer;
    bSet: boolean;
begin
  bSet := false;

  // gets the current coordinates of "p"
  x := 0;
  y := 0;

  for i := 1 to 8 do
    for j := 1 to 8 do
      if aMaze[i, j] = p then
        begin
          x := i;
          y := j;
        end;

  while not bSet do
    begin
      i := Random(10);
      j := Random(10);

      if (aMaze[i, j] = 0)
        and (i <> x)
        and (j <> y) then
        begin
          aMaze[i, j] := p;
          if (x <> 0) and (y <> 0 ) then
            aMaze[x, y] := 0;

          bSet := true;
        end;
    end;
end;





procedure TForm1.Gam_Set_Mode;
{ sets demo/game mode, depending on the value of the iMode variable            }
begin
  case iMode of
    0: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Vid_Show_All(false);
         bCheat := false; Vid_Show_Cheat(bCheat);

         Snd_Silence;

         bController := false;
       end;
    1: begin                                                                    // ACL/2 (ACL button release)
         bCheat := false; Vid_Show_Cheat(bCheat);

         iLoopDemoMode := 1;
         Timer_Demo_Mode.Interval := 450;
         Timer_Demo_Mode.Enabled := true;
       end;
    2: begin                                                                    // SELECT GAME (L1)
         Gam_Stop_All_Timers;
         Vid_Show_All(false);
         bCheat := false; Vid_Show_Cheat(bCheat);

         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[10].Picture;          // L
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 1].Picture;        // 1
       end;
    3: begin                                                                    // SELECT GAME (L2)
         bCheat := false; Vid_Show_Cheat(bCheat);

         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[10].Picture;          // L
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 2].Picture;        // 2
       end;
    4: begin                                                                    // SELECT GAME (L3)
         bCheat := false; Vid_Show_Cheat(bCheat);

         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[10].Picture;          // L
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 3].Picture;        // 3
       end;
    5,                                                                          // GAME (L1)
    6,                                                                          // GAME (L2)
    7: begin                                                                    // GAME (L3)
         Vid_Show_All(false);
         bCheat := false; Vid_Show_Cheat(bCheat);

         Timer_Game_Start.Interval := 1000;
         Timer_Game_Start.Enabled := true;
       end;
    8: begin                                                                    // GAME OVER
         // does nothing...
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Warriors_Cell;
{ sets all variables for the warrior's current position                        }
begin
  Gam_Check_Close_To_Arrow;
  Gam_Check_Close_To_Bat;
  Gam_Check_Close_To_Dragon;
  Gam_Check_Close_To_Hook;
  Gam_Check_Close_To_Pit;

  if bCloseToDragon then
    begin
      bController := false;

      iLoopFoundDragon := 1;
      Timer_Found_Dragon.Interval := 60;
      Timer_Found_Dragon.Enabled := true;
    end
  else
    if bCloseToArrow or bCloseToHook then
      begin
        bController := false;

        iLoopFoundArrowHook := 1;
        Timer_Found_Arrow_Hook.Interval := 60;
        Timer_Found_Arrow_Hook.Enabled := true;
      end
    else
      begin
        im_maze.Visible := true;
        aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
        aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
        Vid_Show_Cursor(iCursor, true);

        bController := true;

        Timer_Score.Interval := 100;
        Timer_Score.Enabled := true;

        if bCloseToBat or bCloseToPit then
          begin
            Timer_Blinking.Interval := 1000;
            Timer_Blinking.Enabled := true;
          end;
      end;

  Vid_Show_Cheat(bCheat);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Bat_MovesTimer(Sender: TObject);
{ bats move, with or without the warrior                                       }
begin
  if bBatHasWarrior then
    begin                                                                       // a bat takes the warrior away to a different cell
      Vid_Show_All(false);
      im_icon_hook.Visible := bHaveHook;
      im_icon_arrow.Visible := bHaveArrow;
      aIm_warrior[1].Visible := true;
      aIm_bat[1].Visible := true;

      Snd_Play(pfs_step_bat);

      iLoopBatWarriorMove := 1;
      Timer_Bat_Warrior_Move.Interval := 315;
      Timer_Bat_Warrior_Move.Enabled := true;
    end
  else
    begin                                                                       // a bat has moved to an empty cell
      Gam_Check_Close_To_Bat;

      im_pit.Visible       := false;
      aIm_bat[1].Visible   := false;
      im_hook.Visible      := false;
      aIm_arrow[2].Visible := false;
      im_dragon.Visible    := false;

      if bCloseToArrow
        or bCloseToBat
        or bCloseToDragon
        or bCloseToHook
        or bCloseToPit then
        begin
          Timer_Blinking.Interval := 500;
          Timer_Blinking.Enabled := true;
        end;

      Vid_Show_Cheat(bCheat);

      bController := true;

      Timer_Score.Interval := 100;
      Timer_Score.Enabled := true;
    end;

  Timer_Bat_Moves.Enabled := false;
end;





procedure TForm1.Timer_Bat_Warrior_MoveTimer(Sender: TObject);
{ bat and warrior move together                                                }
  var
    c, i, j: integer;
begin
  case iLoopBatWarriorMove of
    1: begin aIm_bat[1].Visible := false; aIm_bat[2].Visible := true; aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step_bat); end;
    2: begin aIm_bat[2].Visible := false; aIm_bat[3].Visible := true; aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step_bat); end;
    3: begin aIm_bat[3].Visible := false; aIm_bat[4].Visible := true; aIm_warrior[3].Visible := false; aIm_Warrior[4].Visible := true; Snd_Play(pfs_step_bat); end;
    4: begin aIm_bat[4].Visible := false; aIm_bat[5].Visible := true; aIm_Warrior[4].Visible := false; aIm_warrior[5].Visible := true; Snd_Play(pfs_step_bat); end;
    5: begin aIm_bat[5].Visible := false; aIm_bat[1].Visible := true; aIm_warrior[5].Visible := false; aIm_warrior[1].Visible := true; Snd_Play(pfs_step_bat); end;
    6: begin aIm_bat[1].Visible := false; aIm_bat[2].Visible := true; aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step_bat); end;
    7: begin aIm_bat[2].Visible := false; aIm_bat[3].Visible := true; aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step_bat); end;
    8: begin aIm_bat[3].Visible := false; aIm_bat[4].Visible := true;                                                                  Snd_Play(pfs_step_bat); end;
    9: begin aIm_bat[4].Visible := false; aIm_bat[5].Visible := true;                                                                  Snd_Play(pfs_step_bat); end;
   10: begin
         c := 1;                                                                // moves the bat which got the warrior

         while c <= 4 do                                                        // searches for that specific bat
           begin
             if (rBat[c].x = iWarriorX) and (rBat[c].y = iWarriorY) then
               begin
                 repeat
                   i := Random(10);
                   j := Random(10);
                 until
                   aMaze[i, j] = 0;

                 aMaze[iWarriorX, iWarriorY] := 1;
                 rBat[c].x := i;
                 rBat[c].y := j;
                 aMaze[i, j] := 3;

                 c := 4; { = 5 }
               end;

             Inc(c);
           end;

         // moves the warrior
         repeat
           i := Random(10);
           j := Random(10);
         until
           (i <> iWarriorX) and (j <> iWarriorY);

         aMaze[iWarriorX, iWarriorY] := 0; { previous position }

         iWarriorX := i; { new position }
         iWarriorY := j;

         // checks which kind of cell warrior is moving  to
         case aMaze[iWarriorX, iWarriorY] of
           0: begin iLoopMoveToFree   := 3;         Timer_Move_to_Free.Interval   := 1; Timer_Move_to_Free.Enabled   := true; end;
           2: begin iLoopMoveToPit    := 3;         Timer_Move_To_Pit.Interval    := 1; Timer_Move_To_Pit.Enabled    := true; end;
           3: begin iLoopMoveToBat    := 2; { = 3 } Timer_Move_To_Bat.Interval    := 1; Timer_Move_To_Bat.Enabled    := true; end;
           4: begin iLoopMoveToHook   := 3;         Timer_Move_To_Hook.Interval   := 1; Timer_Move_To_Hook.Enabled   := true; end;
           5: begin iLoopMoveToArrow  := 3;         Timer_Move_To_Arrow.Interval  := 1; Timer_Move_To_Arrow.Enabled  := true; end;
           6: begin iLoopMoveToDragon := 3;         Timer_Move_To_Dragon.Interval := 1; Timer_Move_To_Dragon.Enabled := true; end;
         end; // case.of

         Vid_Show_Cheat(bCheat);

         Timer_Bat_Warrior_Move.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopBatWarriorMove);
end;





procedure TForm1.Timer_BlinkingTimer(Sender: TObject);
{ icons and items blink                                                        }
begin
  if bCloseToPit    then im_pit.Visible       := not im_pit.Visible;
  if bCloseToBat    then aIm_bat[1].Visible   := not aIm_bat[1].Visible;
  if bCloseToHook   then im_hook.Visible      := not im_hook.Visible;
  if bCloseToArrow  then aIm_arrow[1].Visible := not aIm_arrow[1].Visible;
  if bCloseToDragon then im_dragon.Visible    := not im_dragon.Visible;

  Timer_Blinking.Interval := 225;
end;





procedure TForm1.Timer_Demo_ModeTimer(Sender: TObject);
{ short demo animation                                                         }
begin
  case iLoopDemoMode of
    1: begin Vid_Show_All(false); im_maze.Visible := true; aIm_cursor[2].Visible := true; aIm_panel_digit_pos[1].Picture := aIm_panel_letter[0].Picture; aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 0].Picture; end;
    2: begin end;
    3: begin end;
    4: begin Vid_Show_All(false); aIm_warrior[1].Visible := true; end;
    5: begin                      aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; end;
    6: begin                      aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; end;
    7: begin Vid_Show_All(false); im_maze.Visible := true; aIm_cursor[2].Visible := true; aIm_panel_digit_pos[1].Picture := aIm_panel_letter[0].Picture; aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 1].Picture; end;
    8: begin end;
    9: begin end;
   10: begin Vid_Show_All(false);                                  aIm_warrior[1].Visible := true; end;
   11: begin                      aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; end;
   12: begin                      aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; end;
   13: begin Vid_Show_All(false); im_maze.Visible := true; aIm_cursor[2].Visible := true; aIm_panel_digit_pos[1].Picture := aIm_panel_letter[0].Picture; aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 2].Picture; end;
   14: begin end;
   15: begin end;
   16: begin Vid_Show_All(false);                                  aIm_warrior[1].Visible := true; aIm_bat[1].Visible := true;  end;
   17: begin                      aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; aIm_bat[1].Visible := false; aIm_bat[2].Visible := true; end;
   18: begin                      aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; aIm_bat[2].Visible := false; aIm_bat[3].Visible := true; end;
   19: begin                      aIm_warrior[3].Visible := false; aIm_Warrior[4].Visible := true; aIm_bat[3].Visible := false; aIm_bat[4].Visible := true; end;
   20: begin                      aIm_Warrior[4].Visible := false; aIm_warrior[5].Visible := true; aIm_bat[4].Visible := false; aIm_bat[5].Visible := true; end;
   21: begin                      aIm_warrior[5].Visible := false; aIm_warrior[1].Visible := true; aIm_bat[5].Visible := false; aIm_bat[1].Visible := true; end;
   22: begin                      aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; aIm_bat[1].Visible := false; aIm_bat[2].Visible := true; end;
   23: begin                      aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; aIm_bat[2].Visible := false; aIm_bat[3].Visible := true; end;
   24: begin                      aIm_warrior[3].Visible := false; aIm_warrior[6].Visible := true; aIm_bat[3].Visible := false; aIm_bat[4].Visible := true; im_pit.Visible := true; end;
   25: begin                      aIm_warrior[6].Visible := false; aIm_warrior[7].Visible := true; aIm_bat[4].Visible := false; aIm_bat[5].Visible := true; im_dragon.Visible := true; end;
   26: begin                                                                                       aIm_bat[5].Visible := false;                             im_dragon.Visible := false; end;
   27:                                                                                                                                                              im_dragon.Visible := true;
   28:                                                                                                                                                              im_dragon.Visible := false;
   29:                                                                                                                                                              im_dragon.Visible := true;
   30:                                                                                                                                                              im_dragon.Visible := false;
   31:                                                                                                                                                              im_dragon.Visible := true;
   32: begin end;
   33: begin end;
  end; // case..of

  // loops action
  Inc(iLoopDemoMode);

  if iLoopDemoMode = 34 then
    iLoopDemoMode := 1;
end;





procedure TForm1.Timer_Five_SecondsTimer(Sender: TObject);
{ this timer ticks after five seconds the "Attack" button has been pressed     }
{ when in "Game Over" mode. If this is the case, the game mode changes to      }
{ "demo" (iMode = 1)                                                           }
begin
  bFiveSeconds := true;
  iMode := 1; Gam_Set_Mode;
  Timer_Five_Seconds.Enabled := false;
end;





procedure TForm1.Timer_Found_Arrow_HookTimer(Sender: TObject);
{ arrow or hook are on an adjacent cell. beeps. icons start blinking           }
begin
  case iLoopFoundArrowHook of
    1: begin
         Snd_Play(pfs_found);

         Timer_Found_Arrow_Hook.Interval := 1000;
       end;
    2: begin
         im_maze.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
         Vid_Show_Cursor(iCursor, true);

         Timer_Blinking.Interval := 300;
         Timer_Blinking.Enabled := true;

         bController := true;

         Timer_Score.Interval := 100;
         Timer_Score.Enabled := true;

         Timer_Found_Arrow_Hook.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopFoundArrowHook);
end;





procedure TForm1.Timer_Found_DragonTimer(Sender: TObject);
{ dragon is on an adjacent cell. beeps. dragon icon starts blinking            }
begin
  case iLoopFoundDragon of
    1: begin
         Snd_Play(pfs_dragon);

         Timer_Found_Dragon.Interval := 470;
       end;
    2: Snd_Play(pfs_dragon);
    3: Snd_Play(pfs_dragon);
    4: Snd_Play(pfs_dragon);
    5: begin
         if bCloseToHook or bCloseToArrow then
           begin
             iLoopFoundArrowHook := 1;
             Timer_Found_Arrow_Hook.Interval := 370;
             Timer_Found_Arrow_Hook.Enabled := true;
           end
         else
           begin
             im_maze.Visible := true;
             aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
             aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
             Vid_Show_Cursor(iCursor, true);

             Timer_Blinking.Interval := 300;
             Timer_Blinking.Enabled := true;

             bController := true;

             Timer_Score.Interval := 100;
             Timer_Score.Enabled := true;
           end;

         Timer_Found_Dragon.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopFoundDragon);
end;





procedure TForm1.Timer_Game_Over_LoseTimer(Sender: TObject);
{ game is over (lost game)                                                     }
begin
  case iLoopGameOver of
    1: begin
         Snd_Play(pfs_gameover);

         Timer_Game_Over_Lose.Interval := 530;
       end;
    2: begin
         Snd_Play(pfs_gameover);

         Timer_Game_Over_Lose.Interval := 870;
       end;
    3: begin
         iMode := 8;

         Timer_Game_Over_Lose.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameOver);
end;





procedure TForm1.Timer_Game_Over_WinTimer(Sender: TObject);
{ game is over (won game)                                                      }
begin
  case iLoopGameOver of
    1: begin
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1, iScore div 10].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iScore mod 10].Picture;

         iMode := 8;

         Timer_Game_Over_Win.Interval := 1000;
       end;
    2: begin
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[10].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iGameOverMode].Picture;
      end;
  end; // case..of

  // loops action
  Inc(iLoopGameOver);

  if iLoopGameOver = 3 then
    iLoopGameOver := 1;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ sets all variables, inits maze, places items, makes game actually start      }
  var
    c,
    i, j,
    rx, ry,
    em:    integer;
    bDone: boolean;
begin
  // clears maze
  for i := -1 to 10 do
    for j := -1 to 10 do
      aMaze[i, j] := 0;

  // sets warrior to (A,0)
  iWarriorX := 0;
  iWarriorY := 0;
  aMaze[0, 0] := 1;

  // places 12 pits into the maze
  for c := 1 to 12 do
    begin
      bDone := false;

      // attempts to set a pit in (rx,ry)
      while not bDone do
        begin
          repeat
            rx := Random(10);
            ry := Random(10);
          until
            (rx <> 0) and (ry <> 0);

          // checks whether (rx, ry) has any pits in the neighbour
          em := 0;

          for i := rx - 1 to rx + 1 do
            for j := ry - 1 to ry + 1 do
              if aMaze[i, j] < 2 then
                Inc(em);

          // sets (rx,ry) as a pit in case there's 9 free cells with (rx,ry) as the center point
          if em = 9 then
            begin
              aMaze[rx, ry] := 2;
              bDone := true;
            end;
        end;
    end;

  // places 4 bats into the maze
  for c := 1 to 4 do
    begin
      bDone := false;

      while not bDone do
        begin
          i := Random(10);
          j := Random(10);

          if aMaze[i, j] = 0 then
            begin
              rBat[c].x := i;
              rBat[c].y := j;
              aMaze[i, j] := 3;

              bDone := true;
            end;
        end;
    end;

  // places hook (optional), arrow and dragon into the maze
  case iMode of
    5: begin bHaveHook := true; im_icon_hook.Visible := true; end;              // game L1: warrior has hook
    6: begin bHaveHook := false; Gam_Set_Item_Position(4); end;                 // game L2: hook is in the maze
    7:       bHaveHook := false;                                                // game L3: there's no hook at all
  end;

  bHaveArrow := false; Gam_Set_Item_Position(5);
  Gam_Set_Item_Position(6);

  // general settings
  iScore := 0;
  iLoopScore := 1;
  iCursor := 1;
  Gam_Set_Warriors_Cell;

  Timer_Game_Start.Enabled := false;
end;





procedure TForm1.Timer_Move_To_ArrowTimer(Sender: TObject);
{ warrior moves to the cell where's the arrow                                  }
begin
  case iLoopMoveToArrow of
    1: begin aIm_warrior[1].Visible := false;  aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
    2: begin aIm_warrior[2].Visible := false;  aIm_warrior[3].Visible := true; Snd_Play(pfs_step); end;
    3: begin
         aIm_bat[5].Visible := false;
         Snd_Play(pfs_got);

         Timer_Move_To_Arrow.Interval := 780;
       end;
    4: begin
         bHaveArrow := true;
         im_icon_arrow.Visible := true;

         Timer_Move_To_Arrow.Interval := 1000;
       end;
    5: begin
         aIm_warrior[3].Visible := false;
         im_maze.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
         Vid_Show_Cursor(iCursor, true);

         aMaze[iWarriorX, iWarriorY] := 1;
         Gam_Set_Warriors_Cell;

         Timer_Move_To_Arrow.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToArrow);
end;





procedure TForm1.Timer_Move_To_BatTimer(Sender: TObject);
{ warrior moves to the cell where's a bat                                      }
  var
    c, i, j: integer;
begin
 case iLoopMoveToBat of
   1: begin                                                          aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
   2: begin                                                          aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step); end;
   3: begin aIm_bat[1].Visible := true;                                                                                               Snd_Play(pfs_step_bat); end;
   4: begin aIm_bat[1].Visible := false; aIm_bat[2].Visible := true;                                                                  Snd_Play(pfs_step_bat); end;
   5: begin aIm_bat[2].Visible := false; aIm_bat[3].Visible := true;                                                                  Snd_Play(pfs_step_bat); end;
   6: begin aIm_bat[3].Visible := false; aIm_bat[4].Visible := true; aIm_warrior[3].Visible := false; aIm_Warrior[4].Visible := true; Snd_Play(pfs_step_bat); end;
   7: begin aIm_bat[4].Visible := false; aIm_bat[5].Visible := true; aIm_Warrior[4].Visible := false; aIm_warrior[5].Visible := true; Snd_Play(pfs_step_bat); end;
   8: begin aIm_bat[5].Visible := false; aIm_bat[1].Visible := true; aIm_warrior[5].Visible := false; aIm_warrior[1].Visible := true; Snd_Play(pfs_step_bat); end;
   9: begin aIm_bat[1].Visible := false; aIm_bat[2].Visible := true; aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step_bat); end;
  10: begin aIm_bat[2].Visible := false; aIm_bat[3].Visible := true; aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step_bat); end;
  11: begin aIm_bat[3].Visible := false; aIm_bat[4].Visible := true;                                                                  Snd_Play(pfs_step_bat); end;
  12: begin aIm_bat[4].Visible := false; aIm_bat[5].Visible := true;                                                                  Snd_Play(pfs_step_bat); Timer_Move_To_Bat.Interval := 350; end;
  13: begin
        c := 1;                                                                 // moves the bat which got the warrior

        while c <= 4 do                                                           // searches for that specific bat
          begin
            if (rBat[c].x = iWarriorX) and (rBat[c].y = iWarriorY) then
              begin
                repeat
                  i := Random(10);
                  j := Random(10);
                until
                  aMaze[i, j] = 0;

                aMaze[iWarriorX, iWarriorY] := 1;
                rBat[c].x := i;
                rBat[c].y := j;
                aMaze[i, j] := 3;
                c := 4; { = 5 }
              end;

            Inc(c);
          end;

        // moves the warrior
        repeat
          i := Random(10);
          j := Random(10);
        until
          (i <> iWarriorX) and (j <> iWarriorY);

        aMaze[iWarriorX, iWarriorY] := 0; { previous position }

        iWarriorX := i; { new position }
        iWarriorY := j;

        // checks which kind of cell warrior is moving to
        case aMaze[iWarriorX, iWarriorY] of
          0: begin iLoopMoveToFree   := 3;         Timer_Move_to_Free.Interval   := 1; Timer_Move_to_Free.Enabled   := true; end;
          2: begin iLoopMoveToPit    := 3;         Timer_Move_To_Pit.Interval    := 1; Timer_Move_To_Pit.Enabled    := true; end;
          3: begin iLoopMoveToBat    := 2; { = 3 } Timer_Move_To_Bat.Interval    := 1; Timer_Move_To_Bat.Enabled    := true; end;
          4: begin iLoopMoveToHook   := 3;         Timer_Move_To_Hook.Interval   := 1; Timer_Move_To_Hook.Enabled   := true; end;
          5: begin iLoopMoveToArrow  := 3;         Timer_Move_To_Arrow.Interval  := 1; Timer_Move_To_Arrow.Enabled  := true; end;
          6: begin iLoopMoveToDragon := 3;         Timer_Move_To_Dragon.Interval := 1; Timer_Move_To_Dragon.Enabled := true; end;
        end; // case.of

        Timer_Move_To_Bat.Enabled := false;
      end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToBat);
end;





procedure TForm1.Timer_Move_To_DragonTimer(Sender: TObject);
{ warrior moves to the cell where's the dragon                                 }
begin
  case iLoopMoveToDragon of
    1: begin aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
    2: begin aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step); Timer_Move_To_Dragon.Interval := 350;  end;
    3: begin
         aIm_warrior[3].Visible := false;
         aIm_warrior[6].Visible := true;
         aIm_bat[5].Visible := false;
         im_dragon.Visible := true;

         Snd_Play(pfs_pit);

         Timer_Move_To_Dragon.Interval := 1500;
       end;
    4: begin
         iLoopGameOver := 1;
         Timer_Game_Over_Lose.Interval := 340;
         Timer_Game_Over_Lose.Enabled := true;

         Timer_Move_To_Dragon.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToDragon);
end;





procedure TForm1.Timer_Move_to_FreeTimer(Sender: TObject);
{ warrior moves to a free cell                                                 }
begin
  case iLoopMoveToFree of
    1: begin aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
    2: begin aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step); end;
    3: begin
         aIm_warrior[3].Visible := false;
         aIm_bat[5].Visible := false;

         aMaze[iWarriorX, iWarriorY] := 1;
         Gam_Set_Warriors_Cell;

         Timer_Move_to_Free.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToFree);
end;





procedure TForm1.Timer_Move_To_HookTimer(Sender: TObject);
{ warrior moves to the cell where's the hook                                   }
begin
  case iLoopMoveToHook of
    1: begin aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
    2: begin aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step); end;
    3: begin
         aIm_bat[5].Visible := false;

         Snd_Play(pfs_got);

         Timer_Move_To_Hook.Interval := 780; end;
    4: begin
         bHaveHook := true;
         im_icon_hook.Visible := bHaveHook;

         Timer_Move_To_Hook.Interval := 1000;
       end;
    5: begin
         aIm_warrior[3].Visible := false;
         im_maze.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
         Vid_Show_Cursor(iCursor, true);

         aMaze[iWarriorX, iWarriorY] := 1;
         Gam_Set_Warriors_Cell;

         Timer_Move_To_Hook.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToHook);
end;





procedure TForm1.Timer_Move_To_PitTimer(Sender: TObject);
{ warrior moves to the cell where's a pit                                      }
begin
  case iLoopMoveToPit of
    1: begin aIm_warrior[1].Visible := false; aIm_warrior[2].Visible := true; Snd_Play(pfs_step); end;
    2: begin aIm_warrior[2].Visible := false; aIm_warrior[3].Visible := true; Snd_Play(pfs_step); Timer_Move_To_Pit.Interval := 350;  end;
    3: begin
         aIm_warrior[3].Visible := false;
         aIm_warrior[6].Visible := true;
         aIm_bat[5].Visible := false;
         im_pit.Visible := true;

         Snd_Play(pfs_pit);

         Timer_Move_To_Pit.Interval := 1500;
       end;
    4: begin
         aIm_warrior[6].Visible := false;
         aIm_warrior[7].Visible := true;

         if bHaveHook then
           Timer_Move_To_Pit.Interval := 1000
         else
           begin
             iLoopGameOver := 1;
             Timer_Game_Over_Lose.Interval := 340;
             Timer_Game_Over_Lose.Enabled := true;

             Timer_Move_To_Pit.Enabled := false;
           end;
       end;
    5: begin
         bHaveHook := false;
         im_hook.Visible := true;
         im_icon_hook.Visible := false;

         // places a new hook if in game L1/L2 (excluding L3)
         if (iMode = 5) or (iMode = 6) then
           Gam_Set_Item_Position(4);
       end;
    6: begin
         aIm_warrior[7].Visible := false;
         im_pit.Visible := false;
         im_hook.Visible := false;
         im_maze.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
         Vid_Show_Cursor(iCursor, true);

         aMaze[iWarriorX, iWarriorY] := 7; { warrior in a pit }
         Gam_Set_Warriors_Cell;

         Timer_Move_To_Pit.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMoveToPit);
end;





procedure TForm1.Timer_ScoreTimer(Sender: TObject);
{ time passes by, bats move, etc.                                              }
  var
    i, j, c, r: integer;
    bDone: boolean;
begin
  if iLoopScore = 50 then
    begin
      Inc(iScore);

      if iScore = 99 then
        begin                                                                   // game over
          Gam_Stop_All_Timers;

          // display score (99)
          Vid_Show_All(false);
          aIm_panel_digit_pos[1].Picture := aIm_panel_number[1, 9].Picture;
          aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, 9].Picture;

          bController := false;

          iLoopGameOver := 1;
          Timer_Game_Over_Lose.Interval := 1;
          Timer_Game_Over_Lose.Enabled := true;

          Timer_Score.Enabled := false;
        end
      else
        begin                                                                   // moves bat
          iLoopScore := 0;                                                        // resets time counter

          bDone := false;

          bBatHasWarrior := false;

          for c := 1 to 4 do                                                    // each bat
            if Random(5) < 2 then                                                 // moves on a random basis (2 out of 5)
              begin
                r := Random(4) + 1;                                                 // random direction
                i := rBat[c].x;
                j := rBat[c].y;

                case r of
                  1: begin Dec(j); if j < 0 then j := 9; end;
                  2: begin Inc(i); if i > 9 then i := 0; end;
                  3: begin Inc(j); if j > 9 then j := 0; end;
                  4: begin Dec(i); if i < 0 then i := 9; end;
                end; //del case..of

                if aMaze[i, j] < 2 then
                  begin
                    // checks if the warrior was taken by a bat
                    if aMaze[i, j] = 1 then
                      bBatHasWarrior := true;

                    // updates maze
                    aMaze[rBat[c].x, rBat[c].y] := 0;
                    rBat[c].x := i;
                    rBat[c].y := j;
                    aMaze[rBat[c].x, rBat[c].y] := 3;

                    // at least one bat is moving
                    bDone := true;
                  end;
              end;

          // moves bat(s)
          if bDone then
            begin
              Gam_Stop_All_Timers;

              bController := false;

              Snd_Play(pfs_bat);

              Timer_Bat_Moves.Interval := 1300;
              Timer_Bat_Moves.Enabled := true;
            end;
        end;
    end;

  // loops action
  Inc(iLoopScore);
end;





procedure TForm1.Timer_Shoot_ArrowTimer(Sender: TObject);
{ warrior shoots an arrow                                                      }
  var
    i, j: integer;
begin
  case iLoopShootArrow of
    1: begin aIm_arrow[0].Visible := false; aIm_arrow[1].Visible := true; Snd_Play(pfs_arrow); end;
    2: begin aIm_arrow[1].Visible := false; aIm_arrow[2].Visible := true; Snd_Play(pfs_arrow); end;
    3: begin aIm_arrow[2].Visible := false; aIm_arrow[3].Visible := true; Snd_Play(pfs_arrow); end;
    4: begin
         aIm_arrow[3].Visible := false;
         aIm_arrow[4].Visible := true;

         Snd_Play(pfs_arrow);

         i := iWarriorX;
         j := iWarriorY;

         case iCursor of
           1: begin j := iWarriorY - 1; if j < 0 then j := 9; end;
           2: begin i := iWarriorX + 1; if i > 9 then i := 0; end;
           3: begin j := iWarriorY + 1; if j > 9 then j := 0; end;
           4: begin i := iWarriorX - 1; if i < 0 then i := 9; end;
         end; // case..of

         if aMaze[i, j] = 6 then
           begin
             // win!
             im_dragon.Visible := true;

             iLoopShootArrow := 6; { = 7 }
             Timer_Shoot_Arrow.Interval := 630;
           end;
       end;
    5: begin                                                                    // arrow missed to hit the dragon (part 1).
         aIm_arrow[4].Visible := false;
         aIm_arrow[5].Visible := true;

         Snd_Play(pfs_gameover);

         Timer_Shoot_Arrow.Interval := 1500;
       end;
    6: begin                                                                    // arrow missed to hit the dragon (part 2).
         aIm_warrior[0].Visible := false;
         aIm_arrow[5].Visible := false;
         im_maze.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_letter[iWarriorY].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2, iWarriorX].Picture;
         Vid_Show_Cursor(iCursor, true);

         bHaveArrow := false;

         // places a new arrow in the maze
         Gam_Set_Item_Position(5);
         Gam_Set_Item_Position(6);
         Gam_Set_Warriors_Cell;

         Timer_Shoot_Arrow.Enabled := false;
       end;
    7: begin                                                                    // arrow hit the dragon. "win" routine starts
         aIm_arrow[4].Visible := false;
         aIm_arrow[6].Visible := true;
         im_dragon.Visible := false;

         Snd_Play(pfs_win);

         iGameOverMode := iMode - 4; // sets which game was played

         iLoopGameOver := 1;
         Timer_Game_Over_Win.Interval := 5000;
         Timer_Game_Over_Win.Enabled := true;

         Timer_Shoot_Arrow.Enabled := false;
       end
  end; // case..of

  // loops action
  Inc(iLoopShootArrow);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
  pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

  iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
